/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

            v. 1.3
        Web Operating System
*/
/**********************************************
 * Class: OS.box
 **********************************************/
/**********************************************
 * Group: Boxes
 * Creation of a simple GUI box
 * With default theme, it completely fits the container
 *
 * Example:
 * (code)
 *     var box = new OS.box();
 *     box.elements.box.style.right = '50%';
 *     box.addElement('button', 'test_id', 'Test Label');
 *     box.inject(container_id);
 * (end)
 **********************************************/
OS.box = new Class({
    /**********************************************
     * Properties: options
     **********************************************/
    'Implements': Options,
    'options': {
        'type': 'div',
        'className': ''
    },
    'elements': {
        'box': false,
        'container': false
    },
    'childs': [],
    
    /**********************************************
     * Method: initialize
     * Constructor
     **********************************************/
    'initialize': function(options)
    {
        // Merge passed options with default
        this.setOptions(options);
        // Create a box
        this.elements.box = this.createBox();
    },
    
    /**********************************************
     * Method: createBox
     * Create a box container
     **********************************************/
    'createBox': function(type)
    {
        // Creation of the box Element
        var box, id, legend, className;
        id = $time()+'_'+$random(0, 100)+'_gui_box';
        className = type ? 'box '+type+' '+this.options.className : 'box '+this.options.className;
        
        // Div
        if(this.options.type==='div') {
            box = new Element('div', {
                'id': id,
                'class': className
            });
        }
        // Form
        else if(this.options.type==='form') {
            box = new Element('form', {
                'id': id,
                'class': className,
                'method': 'post',
                'action':'',
                'events': {
                    'submit': function(){ return false; }
                }
            });
        }
        // Fieldset
        else if(this.options.type==='fieldset') {
            box = new Element('fieldset', {
                'id': id,
                'class': className
            });
            legend = new Element('legend', {
                'html': this.options.label
            });
            legend.inject(box);
        }
        else {
            box = false;
        }
        return box;
    },
    
    /**********************************************
     * Method: inject
     * Inject gui box in a window
     * 
     * Parameters:
     *     container - Container the box must go to. 
     *          Possible values: HTML Element, 
     *                           GUI Object Instance (to nest), 
     *                           ID of a window (it goes into the id_content), 
     *                           ID of HTML Element
     **********************************************/
    'inject': function(container)
    {
        // Looking for a valid container
        if(container) {
            // Container is an Element
            if($type(container)==='element') {
                this.elements.container = container;
            }
            // Container is a OS.box object instance
            else if(container.elements && container.elements.box) {
                this.elements.container = container.elements.box;
            }
            // Container is a window ID
            else if($(container+'_content')) {
                this.elements.container = $(container+'_content');
            }
            // Container is an ID
            else if($(container)) {
                this.elements.container = $(container);
            }
        }
        // Inject the box Element in the container Element
        if(this.elements.container) {
            this.elements.box.inject(this.elements.container);
        }
        
        // Inject childs into the box
        this.injectChilds(this.elements.box);
        
        return this.elements.container;
    },
    
    /**********************************************
     * Method: addElement
     * Add an OS.elements to the box Elements pool (childs)
     * See elements.js for a complete argument list.
     *
     * Example:
     *  (code)
     *      box.addElement('label', 'label_id', 'Label value');
     *  (end)
     *
     * Parameters:
     *  name - name of the OS.elements to add
     *  specific - Element option from elements.js
     **********************************************/
    'addElement': function(name)
    {
        var arg = $A(arguments);
        arg.shift();
        
        // Get the element(s)
        var fn = OS.elements[name].pass(arg);
        var result = fn();
        
        // Add the result Elements
        if(result) {
            var element = new Element('div', {'class':'x-elem x-elem-'+name});
            result.each(function(res) {
                if($type(res)==='element') {
                    res.inject(element);
                }
            });
            this.childs.push(element);
        }
    },
    
    /**********************************************
     * Method: injectChilds
     * Inject childs from this.childs into this.elements.box
     **********************************************/
    'injectChilds': function(container)
    {
        if(this.childs.length>0) {
            this.childs.each(function(child) {
                if($type(child)==='element') {
                    child.inject(container);
                }
            });
        }  
    }
});
