/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

             v. 1.3
        Web Operating System
*/
/**********************************************
 * Class: OS.elements
 * Dictionnary of GUI Elements methods. It is used
 * in OS.box by the addElement method.
 **********************************************/
OS.elements = {
    /**********************************************
     * Method: table
     * Table
     * 
     * Parameters:
     *     id - id
     *     options - cols, sum, data
     *
     * Example:
     * (start code)
     *  var gui = new this.gui();
     *  gui.table(id, {
     *      sum: [
     *          'col1',
     *          'col2'
     *      ],
     *      data:[
     *          'cell1',
     *          'cell2',
     *          'cell3',
     *          'cell1'
     *      ],
     *      style: 'width: 100%; height: 100%;'
     *  });
     *  gui.inject(id);
     * (end)
     **********************************************/
    'table': function(id, options)
    {
        var sum = options.sum ? options.sum : false;
        var cols = options.cols ? options.cols : sum ? sum.length : 3;
        var data = options.data ? options.data : [];
        var rows = Math.ceil(data.length / cols);
        var style = options.style ? options.style : '';
        var html = '';
        
        //Head
        if(sum && sum.length===cols)
        {
            html += '<thead><tr>';
            for(var i=0; i<cols; i+=1)
            {
                html += '<th>'+sum[i]+'</th>';
            }
            html += '</tr></thead>';
        }
        
        //Fill cells
        var col = 1;
        html += '<tbody>';
        data.each(function(cell, i)
        {
            if(col===1)
            {
                html += '<tr>';
            }
            html += '<td>'+cell+'</td>';
            if(col===cols && i!==(data.length -1))
            {
                html += '</tr>';
                col=0;
            }
            col+=1;
        });
        
        //Fill empty cells
        if(data.length % cols!==0)
        {
            var remains = (cols * rows) - data.length;
            for(var j=0; j<remains; j+=1)
            {
                html += '<td></td>';
            }
        }
        html += '</tr></tbody>';
        
        //Create Element
        var element = new Element('table',
        {
            'id': id+'_gui_table',
            'style': style,
            'html': html
        });
        
        return [element];
    },

    /**********************************************
     * Method: pre
     * Preview, HTML balise <pre/>
     *
     * Parameters:
     *     id - id
     *     value - value inside pre
     *     options - no options
     **********************************************/
    'pre': function(id, options)
    {
        options = options ? options : {};
        var value = options.value ? options.value : '';
        var className = options.className ? options.className : '';
        var element = new Element('pre',
        {
            'id': id+'_gui_pre',
            'class': className,
            'html': value
        });
        return [element];
    },

    /**********************************************
     * Method: label
     * Element texte
     * 
     * Parameters:
     *     id - Element ID
     *     label - text contained in the Element
     *     options - See Options
     *
     * Options:
     *      className - Class to add to the Element
     *      forElem- id of the Element the label is attached to
     **********************************************/
    'label': function(id, label, options)
    {
        options = options ? options : {};
        var className = options.className ? options.className : '';
        var forElem = options.forElem ? options.forElem : '';
        var element = new Element('label',
        {
            'id': id+'_gui_label',
            'html': label,
            'for': forElem
        });
        return [element];
    },

    /**********************************************
     * Method: div
     * DIV Element
     * 
     * Parameters:
     *     id - Element ID
     *     options - See Options
     *
     * Options:
     *      html - Optional content of the DIV
     *      className - Class to add to the Element
     **********************************************/
    'div': function(id, options)
    {
        options = options ? options : {};
        var html = options.html ? options.html : '';
        var className = options.className ? options.className : '';
        var element = new Element('div',
        {
            'id': id+'_gui_div',
            'class': className,
            'html': html
        });
        return [element];
    },

    /**********************************************
     * Method: button
     * Element bouton
     * 
     * Parameters:
     *     id - Element ID
     *     label - texte contenu dans l'element
     *     options - action
     **********************************************/
    'button': function(id, label, options)
    {
        options = options ? options : {};
        var action = options.action ? options.action : function(){ return false; };
        var className = options.className ? options.className : '';
        var style = style ? style : '';
        var element = new Element('button',
        {
            'id': id+'_gui_button',
            'class': className,
            'html': label,
            'style': style,
            'events': {'click': action}
        });
        return [element];
    },
    
    /**********************************************
     * Method: reset
     * 
     * Parameters:
     *     id - id
     *     label - label
     *     options - return
     **********************************************/
    'reset': function(id, label, options)
    {
        options = options ? options : {};
        var element = new Element('input',
        {
            'id': id+'_gui_reset',
            'type': 'reset',
            'value': label
        });
        return [element];
    },

    /**********************************************
     * Method: checkbox
     * Checkbox HTML Element
     * 
     * Parameters:
     *     id - id
     *     name - text in the element
     *     options - action, checked, name_before, class
     *
     * Returns:
     *     The checkbox
     **********************************************/
    'checkbox': function(id, name, options)
    {
        options = options ? options : {};
        var action = options.action ? options.action : function(){};
        var className = options.className ? options.className : '';
        var checked = options.checked ? 'checked' : '';
        var name_before = options.name_before ? options.name_before : false;
        var input = new Element('input',
        {
            'id': id+'_gui_checkbox',
            'type': 'checkbox',
            'class': 'checkbox '+className,
            'checked': checked,
            'events': {'click': action}
        });
        var label = new Element('label',
        {
            'for': id+'_gui_checkbox',
            'html': name
        });
        if(name_before) {
            return [label, input];
        }
        else {
            return [input, label];
        }
        return false;
    },

    /**********************************************
     * Method: radio
     * Radio button element. All related radio
     * buttons must have the same ID's !
     * MUST BE USED IN A FORM ATM !
     * 
     * Parameters:
     *     name - element identifier, must be the same
     *          as all other radio buttons of the choice
     *          see <gui>
     *     value - value of the button
     *     label_html - Label text
     *     options - checked, action, name_before
     **********************************************/
    'radio': function(name, value, label_html, options)
    {
        options = options ? options : {};
        var id = name+'_'+$time()+'_'+$random(0,50)+'_gui_radio';
        var className = options.className ? options.className : '';
        var checked = options.checked ? 'checked' : '';
        var action = options.action ? options.action : function(){};
        var name_before = options.name_before ? options.name_before : false;
        var input = new Element('input',
        {
            'id': id,
            'class': className,
            'name': name+'_gui_radio',
            'value': value,
            'type': 'radio',
            'checked': checked,
            'events': {'click': action}
        });
        var label = new Element('label',
        {
            'for': id,
            'html': label_html
        });
        if(name_before) {
            return [label, input];
        }
        else {
            return [input, label];
        }
        return false;
    },

    /**********************************************
     * Method: radioGet
     * Return the value of a group of radio button
     * (Independent from the instance of OS.gui)
     * 
     * Parameters:
     *     name - radio buttons' name
     *     childOf - parent id of the form
     **********************************************/
    'radioGet': function(name, childOf)
    {
        var value = false;
        var list = $(childOf).getChildren('form')[0][name+'_gui_radio'];
        $A(list).each(function(radio){
            if(radio.checked=='true' || radio.checked)
            {
                value = radio.get('value');
            }
        });
        return value;
    },

    /**********************************************
     * Method: radioSet
     * 
     * Parameters:
     *     value - value
     *     name - radio buttons' name
     *     childOf - parent id of the form
     **********************************************/
    'radioSet': function(value, name, childOf)
    {
        var list = $(childOf).getChildren('form')[0][name+'_gui_radio'];
        $A(list).each(function(radio){
            if(radio.value==value)
            {
                radio.set('checked', 'true');
            }
            else
            {
                radio.set('checked', 'false');
            }
        });
    },

    /**********************************************
     * Method: field
     * Element champs
     * 
     * Parameters:
     *     id - Element ID
     *     options - See Options
     * Options:
     *      value - Initial value of the field
     *      keydown - Action when any key is down
     *      focus - Action on focus, Default: reset field content if
     *              any default value is inside (not user content)
     *      style - Row style to apply
     *      label - Label of the field (click will focus on the field)
     *      size - Apparent size of the field, Default, maxlength or 20
     *      maxlength - Character limitation in the field, Default: 255
     *      className - class(es) to apply to the field
     **********************************************/
    'field': function(id, options)
    {
        options = options ? options : {};
        var style = options.style ? options.style : '';
        var value = options.value ? options.value : '';
        var className = options.className ? options.className : '';
        var maxlength = options.maxlength ? options.maxlength: 255;
        var size = options.size ? options.size: options.maxlength ? options.maxlength: 20;
        var keydown = options.keydown ? options.keydown : function(){};
        var focus = options.focus ? options.focus : function(e) {
            if(value===e.target.value) {
                e.target.value = '';
            }
        };
        var field = new Element('input',
        {
            'id': id+'_gui_field',
            'type': 'text',
            'class': className,
            'style': style,
            'value': value,
            'size': size,
            'maxlength': maxlength,
            'events': {'keydown': keydown, 'focus':focus}
        });
        if(options.label)
        {
            var label = new Element('label',
            {
                'for': id+'_gui_field',
                'html': options.label
            });
            return [label, field];
        }
        else
        {
            return [field];
        }
        return false;
    },

    /**********************************************
     * Method: textarea
     * Element champs multilignes
     * 
     * Parameters:
     *     id - Element ID
     *     options - value, keydown, focus, style, readonly, class
     **********************************************/
    'textarea': function(id, options)
    {
        options = options ? options : {};
        var className = options.className ? options.className : '';
        var value = options.value ? options.value : '';
        var style = options.style ? options.style : '';
        var readonly = options.readonly ? 'readonly' : false;
        var keydown = options.keydown ? options.keydown : function(){};
        var focus = options.focus ? options.focus : function(e) {
            if(value===e.target.value)
            {
                e.target.value = '';
            }
        };
        var label = options.label ? new Element('label',
        {
            'for': id+'_gui_textarea',
            'html': options.label
        }) : new Element('div');
        var text = new Element('textarea',
        {
            'id': id+'_gui_textarea',
            'class': className,
            'html': value,
            'style': style,
            'readonly': readonly,
            'events': {'keydown': keydown, 'focus':focus}
        });
        return [label, text];
    },

    /**********************************************
     * Method: option
     * Element de selecton d'options
     * 
     * Parameters:
     *     id - Element ID
     *     options - value, change, label, options, multiple, value, selected
     **********************************************/
    'option': function(id, options)
    {
        options = options ? options : {};
        var value = options.value ? options.value : '';
        var multiple = options.multiple ? options.multiple : false;
        var selected = options.selected ? options.selected : false;
        var change = options.change ? options.change : function(){};
        var label = options.label ? new Element('label',
        {
            'for': id+'_gui_option',
            'html': options.label
        }) : new Element('div');
        var select = new Element('select',
        {
            'id': id+'_gui_option',
            'multiple': multiple,
            'events': {'change':change}
        });
        var elem;
        options.options.each(function(option)
        {
            if(selected && option[1]===selected)
            {
                elem = new Element('option',
                {
                    'html': option[0],
                    'value': option[1],
                    'selected': 'selected'
                }).inject(select);
            }
            else
            {
                elem = new Element('option',
                {
                    'html': option[0],
                    'value': option[1]
                }).inject(select);
            }
        });
        return [label, select];
    }
};
