﻿/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/
    
             v. 1.3
        Web Operating System
*/
/**********************************************
 * Class: OS.panel
 * GUI Panel, herited from OS.box
 * Handle boxes orientation,separation,nest
 *
 * Examples:
 * (code)
 *     // Order of injection is determinant to get what you want
 *     var id = 'window_id_here'; // See <OS.panel.inject> for possible values
 *     var panel2 = new OS.panel({
 *         'orientation': 'south',
 *         'size': 50
 *     });
 *     panel2.addElement('label', 'panel2', 'Bottom panel');
 *     panel2.inject(id);
 *     
 *     var panel1 = new OS.panel({
 *         'orientation': 'west',
 *         'size': 150
 *     });
 *     panel1.addElement('label', 'panel1', 'Left Panel');
 *     panel1.inject(id);
 *     
 *     var center = new OS.panel();
 *     center.addElement('label', 'panel3', 'Center Panel');
 *     center.inject(id);
 * (end)
 **********************************************/
OS.panel = new Class({
    'Extends': OS.box,
    'options': {
        'orientation': 'center',
        'resizable': false,
        'min': false,
        'max': false,
        'size': false,
        'position': 0,
        'separate': true,
        'separator_size': 3,
        'padding': 5
    },
    'elements': {
        'previous': false,
        'separator': false,
        'separator_ghost': false
    },
    'orientationSettings': {
        'center': {
            'position': 'left',
            'size': 'width',
            'auto': {}
        },
        'west': {
            'position': 'left',
            'size': 'width',
            'auto': {
                'right': 'auto'
            }
        },
        'east': {
            'position': 'right',
            'size': 'width',
            'auto': {
                'left': 'auto'
            }
        },
        'north': {
            'position': 'top',
            'size': 'height',
            'auto': {
                'bottom': 'auto'
            }
        },
        'south': {
            'position': 'bottom',
            'size': 'height',
            'auto': {
                'top': 'auto'
            }
        }
    },
    
    /**********************************************
     * Method: initialize
     * Constructor
     **********************************************/
    'initialize': function(options)
    {
        // Merge passed options with default
        this.setOptions(options);
        // Create a box
        this.elements.box = this.createBox('panel_'+this.options.orientation);
    },
    
    /**********************************************
     * Method: inject
     * Inject gui panel in a window
     * 
     * Parameters:
     *     container - Container the box must go to. 
     *     Possible values: HTML Element, 
     *                      GUI Object Instance (to nest), 
     *                      ID of a window (it goes into the id_content), 
     *                      ID of HTML Element
     **********************************************/
    'inject': function(container)
    {
        // Looking for a valid container Element
        if(container) {
            // Container is an Element
            if($type(container)==='element') {
                this.elements.container = container;
            }
            // Container is an OS.box instance
            else if(container.elements && container.elements.box) {
                this.elements.container = container.elements.box;
            }
            // Container is a window ID
            else if($(container+'_content')) {
                this.elements.container = $(container+'_content');
            }
            // Container is an ID
            else if($(container)) {
                this.elements.container = $(container);
            }
        }
        // Inject the box Element in the container Element
        if(this.elements.container) {
            // Inject childs into the box
            this.injectChilds(this.elements.box);
            // Create separator
            if(this.options.separate) {
                this.separator(this.elements.box);
            }
            // Put the box in the container
            this.elements.box.inject(this.elements.container);
            // Set size and position
            this.setBoxPosition();
            this.adaptSuroundingBoxes();
        }
        return this.elements.container;
    },
    
    /**********************************************
     * Group: Position
     **********************************************/
    /**********************************************
     * Method: setBoxPosition
     * Set position of a box Element.
     * After box injection.
     **********************************************/
    'setBoxPosition': function()
    {
        // Get orientation settings
        var pos = this.orientationSettings[this.options.orientation]['position'];
        var size = this.orientationSettings[this.options.orientation]['size'];
        var setAuto = this.orientationSettings[this.options.orientation]['auto'];
        
        // Set box position from options
        setAuto['padding'] = this.options.padding+'px';
        this.elements.box.setStyle(pos, this.options.position);
        this.elements.box.setStyle(size, this.options.size - (2*this.options.padding));
        this.elements.box.setStyles(setAuto);
        
        // Set separator option
        if(this.elements.separator) {
            setAuto['padding'] = '';
            this.elements.separator.setStyle(pos, (this.options.position + this.options.size));
            this.elements.separator.setStyle(size, this.options.separator_size);
            this.elements.separator.setStyles(setAuto);
        }
    },
    
    /**********************************************
     * Method: adaptSuroundingBoxes
     * Adapt surounding boxes according to their orientation.
     * Order of injection is determinant to get what you want (FIXME?)
     **********************************************/
    'adaptSuroundingBoxes': function()
    {
        var orientation, size, position, padding, value;
        var o = this;
        // Get all boxes in container
        var children = this.elements.container.getChildren('.box');
        children.each(function(child){
            // Other box than this one
            if(child.id != o.elements.box.id) {
                // Adapt with present boxes
                orientation = o.getBoxOrientation(child);
                size = child.getStyle(o.orientationSettings[orientation]['size']).toInt();
                position = child.getStyle(o.orientationSettings[orientation]['position']).toInt();
                padding = child.getStyle('padding').toInt();
                value = position+size+o.options.separator_size+(2*padding)+'px';
                // Set values
                o.elements.box.setStyle(o.orientationSettings[orientation]['position'], value);
                if(o.elements.separator) {
                    o.elements.separator.setStyle(o.orientationSettings[orientation]['position'], value);
                }
            }
        });
    },
    
    /**********************************************
     * Method: getBoxOrientation
     * get orientation of a box Element in the DOM
     **********************************************/
    'getBoxOrientation': function(elem)
    {
        if(elem.hasClass('panel_west')) {
            return 'west';
        }
        else if(elem.hasClass('panel_east')) {
            return 'east';
        }
        else if(elem.hasClass('panel_south')) {
            return 'south';
        }
        else if(elem.hasClass('panel_north')) {
            return 'north';
        }
        return false;
    },
    
    /**********************************************
     * Group: Separator
     **********************************************/
    /**********************************************
     * Method: separator
     * Box horizontal or vertical separator
     **********************************************/
    'separator': function(box)
    {
        var sep = this.createSeparator(box);
        sep.inject(this.elements.container);
        this.elements.separator = sep;
    },

    /**********************************************
     * Method: createSeparator
     * Create separator Element
     *
     * Parameters:
     *  box_id - id of the box Element
     **********************************************/
    'createSeparator': function(box) {
        var className, classResize, classOrientation;
        classResize = this.options.resizable ? 'separator_resizable' : '';
        classOrientation = 'separator_'+this.options.orientation;
        className = classResize+' '+classOrientation;
        return new Element('span', {
            'id': box.id+'_separator',
            'class': className,
            'events': this.createSeparatorEvent()
        });
    },

    /**********************************************
     * Method: createSeparatorEvent
     * Create the Event for resize with the separator
     **********************************************/
    'createSeparatorEvent': function() {
        var o = this;
        var diff, min, max, pos, sep, next, prev;
        return {
            'mousedown': function(e) {
                // Separator clicked
                e.preventDefault();
                sep = o.elements.separator;
                next = sep.getNext('.'+container_id);
                prev = sep.getPrevious('.'+container_id);
                if(next && prev) {
                    win = sep.getParents('div.window')[0];
                    o.elements.separator_ghost = sep.clone().addClass('separator_ghost').inject(sep, 'before');
                    // Set minimum and maximum
                    if(o.options.orientation=='west' || o.options.orientation=='east') {
                        diff = win.getPosition().x + container.getPosition(win).x;
                        min = prev.offsetLeft + min_size;
                        max = next.offsetLeft + next.offsetWidth - max_size;
                    }
                    else {
                        diff = win.getPosition().y + container.getPosition(win).y;
                        min = prev.offsetTop + min_size;
                        max = next.offsetTop + next.offsetHeight - max_size;
                    }
                    // Event on separator move
                    o.elements.container.addEvents(o.createContainerEvent());
                }
            }
        };
    },

    /**********************************************
     * Method: createContainerEvent
     * Create the Event for resize of the container
     **********************************************/
    'createContainerEvent': function() {
        var o = this;
        var pos;
        return {
            'mousemove': function(e) {
                e.preventDefault();
                // Get mouse position
                if(o.options.orientation=='west' || o.options.orientation=='east') {
                    pos = e.client.x - diff;
                }
                else {
                    pos = e.client.y - diff;
                }
                // If position is outside min or max, set it to it :)
                if(pos>o.options.max) {
                    pos = o.options.max;
                }
                else if(pos<o.options.min) {
                    pos = o.options.min;
                }
                // Move the ghost
                if(o.options.orientation=='west' || o.options.orientation=='east') {
                    o.elements.separator_ghost.style.left = pos+'px';
                }
                else {
                    o.elements.separator_ghost.style.top = pos+'px';
                }
            },
            // Set boxes positions when mouse released
            'mouseup': function(e) {
                o.elements.container.removeEvents('mousemove');
                o.elements.container.removeEvents('mouseup');
                o.elements.separator_ghost.destroy();
                //gui.setPositions(pos);
                console.log('set pos now');
            }
        };
    }
});

